#!/bin/bash

usage() {
    cat <<EOF
Usage: $0 [options]

Options:
   -h                   show this help message and exit
   -r RELEASEVER        Value to use for \$releasever (example: epel-5,
                        fedora-14, fedora-rawhide)
   -a ARCH              Value used for \$basearch
   -d RESULTDIR         Directory to store results (default: \$PWD)
   -e ENVR              One or more RPM ENVR's to download from koji
   -t TMPDIR            Temporary directory to use (default: /tmp)
EOF
    exit 1
}

## Detect the system release and version
detect_releasever() {
    RELEASE="fedora"
    VER="rawhide"
    # Guess releasever based on current system release
    if [ -f /etc/fedora-release ]; then
        RELEASE="fedora"
        VER=$(gawk '{print $3}' /etc/fedora-release)
    elif [ -f /etc/redhat-release -a ! -L /etc/redhat-release ]; then
        RELEASE="epel"
        VER=$(gawk '{print $5}' /etc/redhat-release)
        case $VER in 
            4* ) VER=4 ;;
            5* ) VER=5 ;;
            6* ) VER=6 ;;
            * ) VER=6 ;;
        esac
    fi
    echo "$RELEASE-$VER"
}

error() {
    echo "Error: $1"
    exit 1
}

sigterm() {
    cleanup
    exit 1
}

cleanup() {
    test -n "${SETUP}" && rm ${SETUP}
}

# Default values
BASEARCH=$(uname -i)
RELEASEVER=$(detect_releasever)
RESULTSDIR=${PWD}/results/$(date +%Y%m%d)
TMPDIR=/tmp
ENVR=""
MOCKDIR=/etc/mock

# Process arguments
while getopts "ha:r:d:t:e:" option
do
  case ${option} in
    a ) BASEARCH=$OPTARG ;;
    r ) RELEASEVER=$OPTARG ;;
    d ) RESULTSDIR=$OPTARG ;;
    t ) TMPDIR=$OPTARG ;;
    e ) ENVR=$OPTARG ;;
    \?|h ) usage ;;
    * ) usage ;;
  esac
done

# Sanitize input arguments
RELEASE=${RELEASEVER%%-*}
VERSION=${RELEASEVER##*-}
test -d $MOCKDIR || error "Directory $MOCKDIR not found.  Is mock installed?"
# FIXME - sanitize basearch
# FIXME - sanitize releasever

SETUP=$(mktemp ${TMPDIR}/compose_tree-setup.XXXXXX)

# Prepare mock template

# Ensure sigterm is called on exit
trap sigterm SIGTERM

# Install new packages for testing
cat > ${SETUP} << EOF
#!/bin/sh

## Use koji to download updated packages
for E in ${ENVR}
do
    koji download-build --arch noarch --arch ${BASEARCH} \$E
done

## Install updated packages
yum -y localupdate *.rpm

## Make sure the requested packages were installed
rpm -q ${ENVR}
exit \$?

EOF

MOCKCFG="${RELEASEVER}-${BASEARCH}"
MOCK="mock -r ${MOCKCFG}"
let RETVAL=0
${MOCK} --init
let RETVAL+=$?
${MOCK} --install koji pungi spin-kickstarts
let RETVAL+=$?
${MOCK} --copyin ${SETUP} /tmp/setup.sh
let RETVAL+=$?
${MOCK} --chroot "bash /tmp/setup.sh"
let RETVAL+=$?
${MOCK} --chroot "pungi --nosource --nodebuginfo -G -C -B --force --flavor Fedora --name Fedora --ver ${VERSION} -c /usr/share/spin-kickstarts/fedora-install-fedora.ks"
let RETVAL+=$?

# Report the results.  We do all the copies individually in case one of the files doesn't exist.
mkdir -p ${RESULTSDIR}
cp /var/lib/mock/${MOCKCFG}/result/root.log ${RESULTSDIR}
let RETVAL+=$?

for FNAME in boot.iso pxeboot/vmlinuz pxeboot/initrd.img
do
    ${MOCK} --quiet --copyout /${VERSION}/Fedora/${BASEARCH}/os/images/${FNAME} ${RESULTSDIR}
    let RETVAL+=$?
done

# Clean up and exit
cleanup
exit ${RETVAL}
