#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

from autotest_lib.client.bin import utils
import autoqa.koji_utils, autoqa.util
import os
import re
import fnmatch
import rpmUtils.miscutils
from autoqa.repoinfo import repoinfo
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher
from autoqa.config import getbool

class rpmlint(AutoQATest):
    version = 1 # increment if setup() changes

    @ExceptionCatcher()
    def setup(self, **kwargs):
        # Provide a en_US dictionary for the rpmlint enchant test
        utils.system('yum -y install rpmlint hunspell-en')

    @ExceptionCatcher()
    def initialize(self, config, **kwargs):
        super(self.__class__, self).initialize(config)
        self.rpmdir = os.path.join(self.tmpdir, 'rpms')
        os.makedirs(self.rpmdir)

    @ExceptionCatcher()
    def run_once(self, kojitag, **kwargs):
        super(self.__class__, self).run_once()
        if kwargs['hook'] == 'post-koji-build':
            envrs = [kwargs['envr']]
            update_id = kwargs['envr']
        elif kwargs['hook'] == 'post-bodhi-update':
            envrs = kwargs['envrs']
            update_id = kwargs['name'] or kwargs['id']

        self.result = 'PASSED'
        # order for evaluation of final result; higher index means preference
        self.result_order = ('PASSED','INFO','FAILED','ABORTED')
        self.envr_results = {} # results for invidual packages
        self.outputs = []
        self.highlights = []

        koji = autoqa.koji_utils.SimpleKojiClientSession()

        for envr in envrs:
            # add header
            msg = '%s\n%s\n%s' % ('='*40, envr, '='*40)
            print msg
            self.outputs.append(msg)

            # erase old rpm packages from self.rpmdir, needed to cleanup from last run
            rpms = fnmatch.filter(os.listdir(self.rpmdir), '*.rpm')
            print 'Removing all RPMs from %s' % self.rpmdir
            for rpm in rpms:
                os.remove(os.path.join(self.rpmdir, rpm))

            # download packages
            pkgurls = koji.nvr_to_urls(envr)
            print "Saving RPMs to %s" % self.rpmdir
            for p in pkgurls:
                print "Grabbing %s" % p
                localfile = os.path.join(self.rpmdir, os.path.basename(p))
                autoqa.util.grabber.urlgrab(p, localfile)

            # run rpmlint
            cmd = 'rpmlint %s' % self.rpmdir
            cmd_result = utils.run(cmd+" 2>&1", ignore_status=True,
                stdout_tee=utils.TEE_TO_LOGS, stderr_tee=utils.TEE_TO_LOGS)
            outputs = cmd_result.stdout

            # check command return code
            if cmd_result.exit_status == 0:
                result = 'PASSED'
            elif cmd_result.exit_status == 64 or cmd_result.exit_status == 66:
                result = 'FAILED'
            else:
                result = 'ABORTED'

            # TODO filter known/waived errors/warnings
            # - Need to store the filters somewhere authoritative, get signoff, etc

            # extract rpmlint summary line, e.g.:
            # 3 packages and 0 specfiles checked; 9 errors, 80 warnings.
            if result != 'ABORTED':
                match = re.match(r'.*?; ((\d+) errors, (\d+) warnings).', outputs.splitlines()[-1])
                rpmlint_summary = match.group(1)
                warn_count = match.group(3)
                # mark as INFO if some warnings present
                if result == 'PASSED' and int(warn_count) > 0:
                    result = 'INFO'

            # collect output
            self.envr_results[envr] = result
            if self.result_order.index(result) > self.result_order.index(self.result):
                self.result = result
            self.outputs.append(outputs)

            # email results to mailing list and to pkg owner if they optin
            repo = repoinfo.getrepo_by_tag(kojitag)
            pkg_name = rpmUtils.miscutils.splitFilename(envr + '.noarch')[0]
            send_optin_email = getbool(self.config.get('notifications', 'send_optin_email'))
            if repo is not None and send_optin_email and \
            autoqa.util.check_opt_in(pkg_name, repo['collection_name']):
                #FIXME - hardcoded partial email address here - obviously sub-par
                self.mail_to.append('%s-owner@fedoraproject.org' % pkg_name)

        # create self.summary
        if kwargs['hook'] == 'post-koji-build':
            if result != 'ABORTED':
                self.summary = '%s for %s' % (rpmlint_summary, update_id)
            else:
                self.summary = update_id
        elif kwargs['hook'] == 'post-bodhi-update':
            # create result line like "1 PASSED, 2 FAILED, 3 INFO"
            result_count = []
            for res in self.result_order:
                if res in self.envr_results.values():
                    count = len([k for k in self.envr_results.keys() if self.envr_results[k] == res])
                    result_count.append('%d %s' % (count, res))
            result_count = ', '.join(result_count)
            self.summary = '%s for %s' % (result_count, update_id)

